/********************************************************
 * MAT259 PROJ.3  2018                                  *
 *                                                      *
 * Junxiang Yao                                         *     
 *                                                      *
 * Written in Processing 3.3                            *
 *                                                      *
 * External Libraries:                                  *
 *                    PeasyCam                          *
 *                    Computational Geometry            *
 ********************************************************/
import ComputationalGeometry.*;
IsoWrap wrap_wheat;
IsoWrap wrap_normal;


import peasy.*;
import peasy.org.apache.commons.math.*;
import peasy.org.apache.commons.math.geometry.*;

PeasyCam cam;
PeasyDragHandler handler;

PImage map;
PShape globe;
PFont font;
int earth_radius = 400;
boolean show_earth = true;


Table country_information;
Table country_rice_consumption;
Table country_rice_supply;
Table country_wheat_consumption;
Table country_wheat_supply;
Table country_other_cereal_consumption;
Table country_other_cereal_supply;
Table country_vegitable_consumption;
Table country_vegitable_supply;
Table country_oil_consumption;
Table country_oil_supply;
Table country_sugar_consumption;
Table country_sugar_supply;
ArrayList<Country> countries;

boolean draw_line = true;
boolean draw_mesh = false;
boolean draw_pin = false;
boolean draw_self_consumption = false;
boolean hover = false;
int hover_index = 0;
boolean show_individual = false;

//color rice_color = color(138,216,42);
color rice_color = color(35, 123, 23);
color wheat_color = color(128, 81, 14);
//color other_cereal_color = color(35, 123, 23);
//color other_cereal_color = color(13, 151, 127);
color other_cereal_color = color(28, 191, 187);
color vegitable_color = color(221, 60, 56);
//color oil_color = color(91, 71, 255);
color oil_color = color(110, 94, 241);
color sugar_color = color(186, 28, 134);

boolean show_rice = true;
boolean show_wheat = true;
boolean show_other_cereal = true;
boolean show_vegitable = true;
boolean show_oil = true;
boolean show_sugar = true;

int[] mesh_indices = {

  13, 21, 8, 
  13, 8, 10, 
  13, 10, 1, 
  13, 26, 1, 
  26, 19, 1, 
  26, 19, 6, 
  26, 6, 40, 
  40, 6, 23, 
  40, 23, 9, 
  9, 23, 7, 
  9, 7, 38, 
  7, 17, 38, 
  7, 17, 23, 
  23, 6, 17, 
  6, 17, 15, 
  17, 15, 38, 
  15, 20, 6, 
  15, 20, 22, 
  15, 22, 38, 
  38, 22, 2, 
  2, 22, 12, 
  22, 12, 25, 
  22, 25, 20, 
  20, 25, 4, 
  12, 25, 4, 
  20, 5, 4, 
  5, 6, 20, 
  6, 19, 5, 
  19, 5, 16, 
  1, 16, 19, 
  1, 16, 10, 
  16, 10, 37, 
  16, 37, 5, 
  5, 37, 0, 
  4, 5, 0, 
  0, 4, 12, 
  0, 12, 24, 
  0, 37, 24, 
  37, 24, 14, 
  37, 14, 10, 
  10, 14, 8, 
  14, 8, 18, 
  8, 18, 21, 

  3, 18, 11, 
  11, 18, 14, 
  11, 14, 24, 
  11, 24, 12, 
  11, 12, 2, 
  2, 3, 11, 
  2, 3, 38, 


  9, 35, 30, 
  35, 38, 9, 
  9, 40, 30, 
  30, 35, 28, 
  30, 34, 28, 
  30, 34, 27, 
  30, 13, 26, 
  30, 26, 40, 
  13, 30, 27, 
  13, 21, 27, 
  32, 21, 27, 
  27, 34, 32, 
  32, 18, 42, 
  21, 18, 32, 
  18, 42, 3, 
  3, 33, 42, 
  3, 38, 33, 
  38, 33, 35, 
  29, 33, 35, 
  29, 35, 31, 
  28, 31, 35, 
  29, 33, 39, 
  29, 31, 39, 
  28, 31, 39, 
  //28, 31, 41,
  29, 39, 41, 
  29, 33, 41, 
  33, 41, 42, 
  41, 42, 36, 
  36, 39, 41, 
  28, 36, 39, 
  28, 34, 36, 
  32, 34, 36, 
  32, 36, 42
};


PVector[] screenPos;
int horMargin = 100;
int verMargin = 100;

void setup() {
  /***********************
   basic settings
   ************************/
  //size(1250, 720, P3D);
  size(1280, 800, P3D);

  /***********************
   font
   ************************/
  font = createFont("Helvetica", 80);
  textFont(font);

  /***********************
   camera
   ************************/
  cam = new PeasyCam(this, 2500);
  cam.setMinimumDistance(100);
  cam.setMaximumDistance(4500);
  //cam.setLeftDragHandler(handler);

  /***********************
   globe
   ************************/
  //map = loadImage("world.png");
  map = loadImage("world_middle_gray_80.png");
  noStroke();
  globe = createShape(SPHERE, earth_radius); 
  globe.setTexture(map);


  /***********************
   data
   ************************/
  country_information = loadTable("country_information_without_area.csv", "header");
  country_rice_consumption = loadTable("rice_3_without_area.csv", "header");
  country_rice_supply = loadTable("rice_supply.csv", "header");
  country_wheat_consumption = loadTable("wheat_3_without_area.csv", "header");
  country_wheat_supply = loadTable("wheat_supply.csv", "header");
  country_other_cereal_consumption = loadTable("other_cereal_3_without_area.csv", "header");
  country_other_cereal_supply = loadTable("other_cereal_supply.csv", "header");
  country_vegitable_consumption = loadTable("veggies_3_without_area.csv", "header");
  country_vegitable_supply = loadTable("veggies_supply.csv", "header");
  country_oil_consumption = loadTable("oil_3_without_area.csv", "header");
  country_oil_supply = loadTable("oil_supply.csv", "header");
  country_sugar_consumption = loadTable("sugar_3_without_area.csv", "header");
  country_sugar_supply = loadTable("sugar_supply.csv", "header");
  countries = new ArrayList<Country>();
  //country code, name, latitude, longitude, supply
  for (int i = 0; i < country_information.getRowCount(); ++i) {
    countries.add(new Country(i, 
      country_information.getString(i, 0), 
      country_information.getString(i, 1), 
      country_information.getFloat(i, 3), 
      country_information.getFloat(i, 4), 
      country_rice_supply.getFloat(i, 1), 
      country_wheat_supply.getFloat(i, 1), 
      country_other_cereal_supply.getFloat(i, 1), 
      country_vegitable_supply.getFloat(i, 1), 
      country_oil_supply.getFloat(i, 1), 
      country_sugar_supply.getFloat(i, 1)
      ));
  }
  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        //countries.get(i).rice_self_consumption = country_rice_consumption.getFloat(i, j);
        countries.get(i).rice_self_consumption = country_rice_consumption.getFloat(i, j);
      } else if (country_rice_consumption.getFloat(i, j) != 0) {
        countries.get(i).rice_connection.append(j-1);
        countries.get(i).rice_consumption_data.append(country_rice_consumption.getFloat(i, j));
      }
    }
  } 
  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        countries.get(i).wheat_self_consumption = country_wheat_consumption.getFloat(i, j);
      } else if (country_wheat_consumption.getFloat(i, j) != 0) {
        countries.get(i).wheat_connection.append(j-1);
        countries.get(i).wheat_consumption_data.append(country_wheat_consumption.getFloat(i, j));
      }
    }
  } 
  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        countries.get(i).other_cereal_self_consumption = country_other_cereal_consumption.getFloat(i, j);
      } else if (country_other_cereal_consumption.getFloat(i, j) != 0) {
        countries.get(i).other_cereal_connection.append(j-1);
        countries.get(i).other_cereal_consumption_data.append(country_other_cereal_consumption.getFloat(i, j));
      }
    }
  }

  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        countries.get(i).vegitable_self_consumption = country_vegitable_consumption.getFloat(i, j);
      } else if (country_vegitable_consumption.getFloat(i, j) != 0) {
        countries.get(i).vegitable_connection.append(j-1);
        countries.get(i).vegitable_consumption_data.append(country_vegitable_consumption.getFloat(i, j));
      }
    }
  }

  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        countries.get(i).oil_self_consumption = country_oil_consumption.getFloat(i, j);
      } else if (country_oil_consumption.getFloat(i, j) != 0) {
        countries.get(i).oil_connection.append(j-1);
        countries.get(i).oil_consumption_data.append(country_oil_consumption.getFloat(i, j));
      }
    }
  }

  for (int i = 0; i < countries.size(); ++i) {
    for (int j = 1; j < countries.size()+1; ++j) {
      if (i == (j-1)) {
        countries.get(i).sugar_self_consumption = country_sugar_consumption.getFloat(i, j);
      } else if (country_sugar_consumption.getFloat(i, j) != 0) {
        countries.get(i).sugar_connection.append(j-1);
        countries.get(i).sugar_consumption_data.append(country_sugar_consumption.getFloat(i, j));
      }
    }
    //print(i + ":" + countries.get(i).sugar_connection.size() + "\n");
  }
  //print(countries.get(0).name + ", " + countries.get(0).wheat_self_consumption + ", ");
  print(countries.get(0).other_cereal_self_consumption + ", " + countries.get(0).wheat_self_consumption + ", ");

  screenPos = new PVector[countries.size()];

  /***********************
   mesh
   ************************/
  wrap_normal = new IsoWrap(this);
  for (int i = 0; i < country_information.getRowCount(); ++i) {
    wrap_normal.addPt( new PVector(countries.get(i).x, countries.get(i).y, countries.get(i).z));
  }
}

void draw() {
  lights();
  pointLight(102, 102, 102, 0, 0, 1000);
  smooth();
  rotateX(+.2);
  rotateY(-.5);
  colorMode(RGB, 255, 255, 255);
  background(48, 42, 56);
  //background(117,101,133);


  float stroke_buffer;
  for (int i = 0; i < countries.size(); ++i) {
    // mark all the country on the globe
    countries.get(i).draw_location();
    if (countries.get(i).should_display) {
      //draw lines connecting countries
      if (show_rice) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).rice_connection.size(); ++j) {
            stroke_buffer = countries.get(i).rice_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(rice_color, stroke_buffer);
            countries.get(i).draw_rice_connection(countries.get(countries.get(i).rice_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(rice_color);
          countries.get(i).draw_rice_self_consumption_();
        }
      }
      if (show_wheat) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).wheat_connection.size(); ++j) {
            stroke_buffer = countries.get(i).wheat_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(wheat_color, stroke_buffer);
            countries.get(i).draw_wheat_connection(countries.get(countries.get(i).wheat_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(wheat_color);
          countries.get(i).draw_wheat_self_consumption_();
        }
      }
      if (show_other_cereal) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).other_cereal_connection.size(); ++j) {
            stroke_buffer = countries.get(i).other_cereal_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(other_cereal_color, stroke_buffer);
            countries.get(i).draw_other_cereal_connection(countries.get(countries.get(i).other_cereal_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(other_cereal_color);
          countries.get(i).draw_other_cereal_self_consumption_();
        }
      }
      if (show_vegitable) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).vegitable_connection.size(); ++j) {
            stroke_buffer = countries.get(i).vegitable_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(vegitable_color, stroke_buffer);
            countries.get(i).draw_vegitable_connection(countries.get(countries.get(i).vegitable_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(vegitable_color);
          countries.get(i).draw_vegitable_self_consumption_();
        }
      }
      if (show_oil) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).oil_connection.size(); ++j) {
            stroke_buffer = countries.get(i).oil_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(oil_color, stroke_buffer);
            countries.get(i).draw_oil_connection(countries.get(countries.get(i).oil_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(oil_color);
          countries.get(i).draw_oil_self_consumption_();
        }
      }
      if (show_sugar) {
        if (draw_line) {
          for (int j = 0; j < countries.get(i).sugar_connection.size(); ++j) {
            stroke_buffer = countries.get(i).sugar_consumption_data.get(j);
            stroke_buffer = map(stroke_buffer, 0, 50, 50, 255);
            stroke(sugar_color, stroke_buffer);
            countries.get(i).draw_sugar_connection(countries.get(countries.get(i).sugar_connection.get(j)));
          }
        }
        //draw self consumption
        if (draw_self_consumption) {
          fill(sugar_color);
          countries.get(i).draw_sugar_self_consumption_();
        }
      }
    }
  }


  //draw assist line
  if (draw_pin) {
    if (show_rice) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).rice_x, countries.get(i).rice_y, countries.get(i).rice_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).rice_connection.size(); ++j)
              line(countries.get(countries.get(i).rice_connection.get(j)).rice_x, 
                countries.get(countries.get(i).rice_connection.get(j)).rice_y, 
                countries.get(countries.get(i).rice_connection.get(j)).rice_z, 
                countries.get(countries.get(i).rice_connection.get(j)).x, 
                countries.get(countries.get(i).rice_connection.get(j)).y, 
                countries.get(countries.get(i).rice_connection.get(j)).z);
          }
        }
      }
    }
    if (show_wheat) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).wheat_x, countries.get(i).wheat_y, countries.get(i).wheat_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).wheat_connection.size(); ++j)
              line(countries.get(countries.get(i).wheat_connection.get(j)).wheat_x, 
                countries.get(countries.get(i).wheat_connection.get(j)).wheat_y, 
                countries.get(countries.get(i).wheat_connection.get(j)).wheat_z, 
                countries.get(countries.get(i).wheat_connection.get(j)).x, 
                countries.get(countries.get(i).wheat_connection.get(j)).y, 
                countries.get(countries.get(i).wheat_connection.get(j)).z);
          }
        }
      }
    }
    if (show_other_cereal) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).other_cereal_x, countries.get(i).other_cereal_y, countries.get(i).other_cereal_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).other_cereal_connection.size(); ++j)
              line(countries.get(countries.get(i).other_cereal_connection.get(j)).other_cereal_x, 
                countries.get(countries.get(i).other_cereal_connection.get(j)).other_cereal_y, 
                countries.get(countries.get(i).other_cereal_connection.get(j)).other_cereal_z, 
                countries.get(countries.get(i).other_cereal_connection.get(j)).x, 
                countries.get(countries.get(i).other_cereal_connection.get(j)).y, 
                countries.get(countries.get(i).other_cereal_connection.get(j)).z);
          }
        }
      }
    }
    if (show_vegitable) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).vegitable_x, countries.get(i).vegitable_y, countries.get(i).vegitable_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).vegitable_connection.size(); ++j) {
              line(countries.get(countries.get(i).vegitable_connection.get(j)).vegitable_x, 
                countries.get(countries.get(i).vegitable_connection.get(j)).vegitable_y, 
                countries.get(countries.get(i).vegitable_connection.get(j)).vegitable_z, 
                countries.get(countries.get(i).vegitable_connection.get(j)).x, 
                countries.get(countries.get(i).vegitable_connection.get(j)).y, 
                countries.get(countries.get(i).vegitable_connection.get(j)).z);
            }
          }
        }
      }
    }
    if (show_oil) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).oil_x, countries.get(i).oil_y, countries.get(i).oil_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).oil_connection.size(); ++j) {
              line(countries.get(countries.get(i).oil_connection.get(j)).oil_x, 
                countries.get(countries.get(i).oil_connection.get(j)).oil_y, 
                countries.get(countries.get(i).oil_connection.get(j)).oil_z, 
                countries.get(countries.get(i).oil_connection.get(j)).x, 
                countries.get(countries.get(i).oil_connection.get(j)).y, 
                countries.get(countries.get(i).oil_connection.get(j)).z);
            }
          }
        }
      }
    }
    if (show_sugar) {
      for (int i = 0; i < countries.size(); ++i) {
        if (countries.get(i).should_display) {
          stroke(100, 100, 100, 255);
          line(countries.get(i).sugar_x, countries.get(i).sugar_y, countries.get(i).sugar_z, 
            countries.get(i).x, countries.get(i).y, countries.get(i).z);
          if (!(show_individual&&draw_self_consumption)) {
            for (int j = 0; j < countries.get(i).sugar_connection.size(); ++j) {
              line(countries.get(countries.get(i).sugar_connection.get(j)).sugar_x, 
                countries.get(countries.get(i).sugar_connection.get(j)).sugar_y, 
                countries.get(countries.get(i).sugar_connection.get(j)).sugar_z, 
                countries.get(countries.get(i).sugar_connection.get(j)).x, 
                countries.get(countries.get(i).sugar_connection.get(j)).y, 
                countries.get(countries.get(i).sugar_connection.get(j)).z);
            }
          }
        }
      }
    }
  }

  /*********************
   globe
   *********************/
  colorMode(RGB, 255, 255, 255);
  noStroke();
  texture(map);
  if (show_earth) {
    shape(globe);
  }

  /*********************
   mesh
   *********************/
  if (draw_mesh && !show_individual) {
    //wrap_normal.plot();
    //for (int i = 0; i < countries.size(); ++i) {
    //    for (int j = 0; j < countries.size(); ++j) {
    //      if(i!=j){
    //        stroke(255);
    //        countries.get(i).draw_connection(countries.get(j));
    //      }
    //   }
    //}
    if (show_sugar) {
      fill(sugar_color, 100);
      //noStroke();
      stroke(sugar_color, 100);
      draw_sugar_mesh();
    }

    if (show_oil) {
      fill(oil_color, 100);
      //noStroke();
      stroke(oil_color, 100);
      draw_oil_mesh();
    }


    if (show_rice) {
      fill(rice_color, 100);
      //noStroke();
      stroke(rice_color, 100);
      draw_rice_mesh();
    }

    if (show_other_cereal) {
      fill(other_cereal_color, 100);
      //noStroke();
      stroke(other_cereal_color, 100);
      draw_other_cereal_mesh();
    }

    if (show_wheat) {
      fill(wheat_color, 100);
      //noStroke();
      stroke(wheat_color, 100);
      draw_wheat_mesh();
    }

    if (show_vegitable) {
      fill(vegitable_color, 100);
      //noStroke();
      stroke(vegitable_color, 100);
      draw_vegitable_mesh();
    }
  }

  /*********************
   Text
   *********************/
  for (int i=0; i<countries.size(); i++) {
    screenPos[i]= new PVector(screenX(countries.get(i).x, countries.get(i).y, countries.get(i).z), 
      screenY(countries.get(i).x, countries.get(i).y, countries.get(i).z));
  }

  /************************************
   Hover Effect
   ************************************/
  float min_distance = 100000;
  for (int i=0; i<countries.size(); i++) {
    float distance = dist(mouseX, mouseY, screenPos[i].x, screenPos[i].y);
    // Detect if the distance is small enough to trigger the hover effect
    if (distance<20) {
      // if the distance is small enough
      hover = true;
      // if the distance is the smallest among all the candidates distance
      if (min_distance > distance) {
        min_distance = distance;
        hover_index = i;
      }
    }
  }
  if (hover) {
    pushMatrix();
    rotateY(HALF_PI * -1);
    rotateY(countries.get(hover_index).longitude);
    rotateX(countries.get(hover_index).latitude);
    translate(0, 0, earth_radius);
    //translate(countries.get(i).x, countries.get(i).y, countries.get(i).z);
    fill(255, 255, 255);
    //fill(239, 108, 49);
    noStroke();
    textSize(20);
    textAlign(CENTER);
    text(countries.get(hover_index).name, 0, 0, 0);
    popMatrix();
  }

  hover = false;
  noLights();
  gui();
}


void draw_rice_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).rice_x, countries.get(mesh_indices[i]).rice_y, countries.get(mesh_indices[i]).rice_z);
    vertex(countries.get(mesh_indices[i+1]).rice_x, countries.get(mesh_indices[i+1]).rice_y, countries.get(mesh_indices[i+1]).rice_z);
    vertex(countries.get(mesh_indices[i+2]).rice_x, countries.get(mesh_indices[i+2]).rice_y, countries.get(mesh_indices[i+2]).rice_z);
    endShape();
  }
}

void draw_wheat_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).wheat_x, countries.get(mesh_indices[i]).wheat_y, countries.get(mesh_indices[i]).wheat_z);
    vertex(countries.get(mesh_indices[i+1]).wheat_x, countries.get(mesh_indices[i+1]).wheat_y, countries.get(mesh_indices[i+1]).wheat_z);
    vertex(countries.get(mesh_indices[i+2]).wheat_x, countries.get(mesh_indices[i+2]).wheat_y, countries.get(mesh_indices[i+2]).wheat_z);
    endShape();
  }
}

void draw_other_cereal_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).other_cereal_x, countries.get(mesh_indices[i]).other_cereal_y, countries.get(mesh_indices[i]).other_cereal_z);
    vertex(countries.get(mesh_indices[i+1]).other_cereal_x, countries.get(mesh_indices[i+1]).other_cereal_y, countries.get(mesh_indices[i+1]).other_cereal_z);
    vertex(countries.get(mesh_indices[i+2]).other_cereal_x, countries.get(mesh_indices[i+2]).other_cereal_y, countries.get(mesh_indices[i+2]).other_cereal_z);
    endShape();
  }
}

void draw_vegitable_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).vegitable_x, countries.get(mesh_indices[i]).vegitable_y, countries.get(mesh_indices[i]).vegitable_z);
    vertex(countries.get(mesh_indices[i+1]).vegitable_x, countries.get(mesh_indices[i+1]).vegitable_y, countries.get(mesh_indices[i+1]).vegitable_z);
    vertex(countries.get(mesh_indices[i+2]).vegitable_x, countries.get(mesh_indices[i+2]).vegitable_y, countries.get(mesh_indices[i+2]).vegitable_z);
    endShape();
  }
}

void draw_oil_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).oil_x, countries.get(mesh_indices[i]).oil_y, countries.get(mesh_indices[i]).oil_z);
    vertex(countries.get(mesh_indices[i+1]).oil_x, countries.get(mesh_indices[i+1]).oil_y, countries.get(mesh_indices[i+1]).oil_z);
    vertex(countries.get(mesh_indices[i+2]).oil_x, countries.get(mesh_indices[i+2]).oil_y, countries.get(mesh_indices[i+2]).oil_z);
    endShape();
  }
}

void draw_sugar_mesh() {
  for (int i = 0; i < mesh_indices.length; i += 3) {
    beginShape();
    vertex(countries.get(mesh_indices[i]).sugar_x, countries.get(mesh_indices[i]).sugar_y, countries.get(mesh_indices[i]).sugar_z);
    vertex(countries.get(mesh_indices[i+1]).sugar_x, countries.get(mesh_indices[i+1]).sugar_y, countries.get(mesh_indices[i+1]).sugar_z);
    vertex(countries.get(mesh_indices[i+2]).sugar_x, countries.get(mesh_indices[i+2]).sugar_y, countries.get(mesh_indices[i+2]).sugar_z);
    endShape();
  }
}